/*
 * $Id: nccf_def_coord_from_file.c 757 2011-05-13 22:37:50Z pletzer $
 *
 */

#include "nccf_coord.h"
#include <netcdf.h>
#include <string.h>
#include <stdlib.h>
#include "nccf_varObj.h"

/**
 * \ingroup gs_coord_grp
 * Define (construct) a coordinate object from a netcdf file.
 *
 * \param filename name of the netcdf file
 * \param coord_name name of coordinate in file
 * \param coordid (output) ID
 * \return NC_NOERR on success
 *
 * \author Alexander Pletzer and David Kindig, Tech-X Corp.
 */
int nccf_def_coord_from_file(const char *filename,
              const char *coord_name,
              int *coordid){

  int totError = NC_NOERR;
  int ncid, status = NC_NOERR;
  struct nccf_var_obj *v;

  // read the variable
  status = nc_open(filename, NC_NOWRITE, &ncid);
  if( status ) return status;
  totError += abs(status);
  nccf_varCreateFromFile(&v, coord_name, ncid, 1);
  status = nc_close(ncid);
  totError += abs(status);

  // build the object
  int ndims, i;
  int *dims;

  const int save = 1;
  const char *name;
  const char *standard_name;
  const char *units;

  nc_type dataType;

  void *data;

  nccf_varGetNumDims(&v, &ndims);
  nccf_varGetDimsPtr(&v, &dims);
  const char *dimnames[ndims];

  for( i = 0; i < ndims; i++ ) {
    nccf_varGetDimNamePtr(&v, i, &dimnames[i]);
  }

  /* Read the rest of the variable */
  nccf_varGetVarNamePtr(&v, &name);
  nccf_varGetAttribPtr(&v, CF_ATTNAME_STANDARD_NAME, 
		       (const void **) &standard_name);
  nccf_varGetAttribPtr(&v, CF_ATTNAME_UNITS, 
		       (const void **) &units);
  nccf_varGetDataType(&v, &dataType);
  nccf_varGetDataPtr(&v, &data);
  if (dataType == NC_DOUBLE) {
    status = nccf_def_coord(ndims, dims, (const char**)dimnames, 
             (double *) data, save, name, standard_name, units, coordid );
    totError += abs(status);
  }
  else {
    totError += 1; // error
  }

  // clean up
  nccf_varDestroy(&v);

  return totError;
}
