/*
 * API to define a structured grid for the gridspec convention to libcf.
 *
 * $Id: nccf_free_coord.c 737 2011-05-06 17:48:14Z edhartnett $
 * */

#include "nccf_coord.h"
#include <stdio.h>
#include <stdlib.h>

#include <netcdf.h>
#include <nccf_varObj.h>

/**
 * \ingroup gs_coord_grp
 * Free object (destructor).
 *
 * \param coordid coordinate ID
 * \return NC_NOERR on success
 *
 * \author Alexander Pletzer and David Kindig, Tech-X Corp.
 */
int nccf_free_coord(int coordid) {

    struct nccf_coord_type *self;
    self = nccf_li_remove(&CFLIST_COORDINATE, coordid);
    if( nccf_li_get_nelem( &CFLIST_COORDINATE ) == 0 )
        nccf_li_del( &CFLIST_COORDINATE );

    /* free memory */
    free(self->dims);
    free( self->coord_name );

    if (self->save) {
      free(self->data);
    }

    nccf_varDestroy(&self->coordVar);

    free(self);

    return NC_NOERR;
}
