/* $Id: nccf_is_coord.c 804 2011-09-12 03:59:08Z pletzer $ */

#include "nccf_coord.h"
#include <string.h>

/**
 * \ingroup gs_coord_grp
 * Check if coordinate is a longitude 
 * 
 * \param coordid coordinate ID
 * \param yesno (output) 1=yes 0=no
 */
int nccf_is_coord_lon(int coordid, int *yesno) {
  *yesno = 0;
  struct nccf_coord_type *self;
  self = nccf_li_find(&CFLIST_COORDINATE, coordid);
  const char *standard_name;
  const char *units;
  nccf_varGetAttribPtr(&self->coordVar, CF_ATTNAME_STANDARD_NAME,
		       (const void **) &standard_name);
  nccf_varGetAttribPtr(&self->coordVar, CF_ATTNAME_UNITS,
		       (const void **) &units);
  if (strstr(units, "degrees_east") || strstr(units, "degree_east") || 
      strstr(units, "degrees_E") || strstr(units, "degree_E") ||
      strstr(units, "degreesE") || strstr(units, "degreeE") ||
      strstr(standard_name, "longitude")) {
    *yesno = 1;
  }
  return NC_NOERR;
}

/**
 * \ingroup gs_coord_grp
 * Check if coordinate is a latitude 
 * 
 * \param coordid coordinate ID
 * \param yesno (output) 1=yes 0=no
 */
int nccf_is_coord_lat(int coordid, int *yesno) {
  *yesno = 0;
  struct nccf_coord_type *self;
  self = nccf_li_find(&CFLIST_COORDINATE, coordid);
  const char *standard_name;
  const char *units;
  nccf_varGetAttribPtr(&self->coordVar, CF_ATTNAME_STANDARD_NAME,
		       (const void **) &standard_name);
  nccf_varGetAttribPtr(&self->coordVar, CF_ATTNAME_UNITS,
		       (const void **) &units);
  if (strstr(units, "degrees_north") || strstr(units, "degree_north") || 
      strstr(units, "degrees_N") || strstr(units, "degree_N") ||
      strstr(units, "degreesN") || strstr(units, "degreeN") ||
      strstr(standard_name, "latitude")) {
    *yesno = 1;
  }
  return NC_NOERR;
}
