/*
 * API to free a structured grid object for the gridspec convention to
 * libcf.
 *
 * $Id: nccf_free_grid.c 738 2011-05-06 22:26:09Z edhartnett $
 * */

#include "nccf_grid.h"
#include <stdio.h>
#include <stdlib.h>

#include <nccf_varObj.h>

/**
 * \ingroup gs_grid_grp
 * Free structured grid object.
 *
 * \param gridid grid ID
 * \return NC_NOERR on success
 *
 * \author David Kindig and Alexander Pletzer, Tech-X Corp.
 */
int nccf_free_grid(int gridid) {

    struct nccf_struct_grid_type *self;
    self = nccf_li_remove(&CFLIST_STRUCTURED_GRID, gridid);
    if( nccf_li_get_nelem( &CFLIST_STRUCTURED_GRID ) == 0 )
        nccf_li_del( &CFLIST_STRUCTURED_GRID );

    /* call free below */
    if( self->gridname ){
     free( self->gridname );
    }
    if( self->coordids ) free( self->coordids );

    if (self->imask) {
      free(self->imask);
    }

    /* Destroy object */
    free( self );

    return NC_NOERR;
}
