/*
 * $Id: nccf_free_mosaic.c 744 2011-05-09 20:05:05Z dkindig $
 *
 */

#include <nccf_mosaic.h>
#include <stdlib.h>

/**
 * \ingroup gs_mosaic_grp
 * Free (destroy) a mosaic object (this will reclaim memory).
 *
 * \param mosaicid mosaic identifier
 * \return NC_NOERR on success
 *
 * \author David Kindig and Alexander Pletzer, Tech-X Corp.
 */
int nccf_free_mosaic(int mosaicid) {
  
  struct nccf_mosaic_type *self;
  int index;

  /* Remove from list */
  self = nccf_li_remove(&CFLIST_MOSAIC, mosaicid);
  if( nccf_li_get_nelem( &CFLIST_MOSAIC ) == 0 )
      nccf_li_del(&CFLIST_MOSAIC);

  /* Free members */
  if (self->name) free( self->name );
  if (self->gridids) free( self->gridids );

  if (self->gs_slice_format) free(self->gs_slice_format);

  /* Free the list members */
  /* Coordinate names */ 
  nccf_li_begin( &self->coordnameslist );
  while( nccf_li_next( &self->coordnameslist )){
    index = nccf_li_get_id( &self->coordnameslist );
    char *val = nccf_li_remove( &self->coordnameslist, index );
    if( val ) free( val );
  }

  /* Grid names */ 
  nccf_li_begin( &self->gridnameslist );
  while( nccf_li_next( &self->gridnameslist )){
    index = nccf_li_get_id( &self->gridnameslist );
    char *val = nccf_li_remove( &self->gridnameslist, index );
    if( val ) free( val );
  }

  /* Contact index maps */ 
  nccf_li_begin( &self->contactindexlist );
  while( nccf_li_next( &self->contactindexlist )){
    index = nccf_li_get_id( &self->contactindexlist );
    char *val = nccf_li_remove( &self->contactindexlist, index );
    if( val ) free( val );
  }

  /* Grid to grid contacts */ 
  nccf_li_begin( &self->gridtogridlist );
  while( nccf_li_next( &self->gridtogridlist )){
    index = nccf_li_get_id( &self->gridtogridlist );
    char *val = nccf_li_remove( &self->gridtogridlist, index );
    if( val ) free( val );
  }

  /* Delete the head of each list */
  nccf_li_del( &self->coordnameslist );
  nccf_li_del( &self->gridnameslist );
  nccf_li_del( &self->contactindexlist );
  nccf_li_del( &self->gridtogridlist );

  /* Free variable */
  nccf_varDestroy(&self->contactIndex);
  nccf_varDestroy(&self->gridToGrid);
  nccf_varDestroy(&self->coordnames);
  nccf_varDestroy(&self->gridNames);
  
  /* Free object */
  free(self);

  return NC_NOERR;
}
