/**
 * $Id: nccf_def_regrid_from_file.c 787 2011-07-28 19:04:22Z dkindig $
 */

#include <nccf_regrid.h>
#include <netcdf.h>
#include <string.h>
#include <stdio.h>
#include <nccf_varObj.h>
#include <nccf_coord.h>
#include <nccf_grid.h>

int nccf_get_weights( int regrid_id, double **weights ){
  
  double **data;
  struct nccf_var_obj *self;
  self = nccf_li_find(&CFLIST_REGRID, regrid_id);

  nccf_varGetDataPtr( &self, ( void** ) &data );

  return NC_NOERR;

}

/**
 * \ingroup gs_regrid_grp
 * Create a regrid object from a file
 * 
 * /param filename File to read from
 * /param ndims Dimensions of grid
 * /param regrid_id regrid object to write
 * /return NC_NOERR on success
 *
 * \author Alexander Pletzer, Dave Kindig, Tech-X Corp.
 */
int nccf_def_regrid_from_file(const char *filename,
                   int *regridId) {

  int ncid;
  int status = NC_NOERR;
  int *dims;

  struct nccf_regrid_type *self;
  self = ( struct nccf_regrid_type* )malloc( sizeof( struct nccf_regrid_type ));

  /* initialize self */
  nccf_li_new( &self->box_lohi );

  self->weights = NULL;
  self->indices = NULL;
  self->inside_domain = NULL;
  self->nvalid = 0;
  const int *nvalid = NULL;

  status = nc_open( filename, NC_NOWRITE, &ncid );
  if( status ) return status;

  nccf_varCreateFromFile( &self->weights_stt, "weights", ncid, 1 );
  nccf_varCreateFromFile( &self->indices_stt, "indices", ncid, 1 );
  nccf_varCreateFromFile( &self->inside_domain_stt, "inside_domain", ncid, 1 );

  nccf_varGetDimsPtr( &self->indices_stt, &dims );
  self->ntargets = dims[0];
  self->nnodes = dims[1];

  nccf_varGetNumDims( &self->indices_stt, &self->ndims );
  nccf_varGetDimsPtr( &self->indices_stt, &dims );

  nccf_varGetAttribPtr( &self->inside_domain_stt, "nvalid", &nvalid );
  self->nvalid = *nvalid;

  /* Get the weights for use in nccf_apply_regrid */
//  self->weights = (double*)malloc( self->ntargets * self->nnodes * sizeof(double));
//  self->indices = (int*)malloc( self->ntargets * self->nnodes * sizeof(int));
//  self->inside_domain = (int*)malloc( self->ntargets * sizeof(int));

  nccf_varGetDataPtr( &self->weights_stt,       ( void** ) &self->weights );
  nccf_varGetDataPtr( &self->indices_stt,       ( void** ) &self->indices );
  nccf_varGetDataPtr( &self->inside_domain_stt, ( void** ) &self->inside_domain );

  if (( status = nc_close( ncid ))) ERR;

  /* Add the objet to the list */
  if (CFLIST_REGRID == NULL) nccf_li_new(&CFLIST_REGRID);
  *regridId = nccf_li_add(&CFLIST_REGRID, self);

  return NC_NOERR;

}
