/*
 * API to perform regridding operations on structured grids in libcf.
 *
 * $Id: nccf_regrid.h 834 2011-09-14 21:21:07Z pletzer $
 *
 * \author Alexander Pletzer, Tech-X Corp.
 */

#ifndef _NCCF_REGRID_H
#define _NCCF_REGRID_H

#include <libcf_src.h>
#include <cflistitem.h>

/* Variable names we have picked */
#define CF_REGRID_WEIGHTS            "gridspec_weights"
#define CF_REGRID_INDICES            "gridspec_indices"
#define CF_REGRID_INSIDE_DOMAIN      "gridspec_inside_domain"

extern struct CFLISTITEM *CFLIST_REGRID;

struct nccf_regrid_type {

   /* list of boxes and their lo/hi sets of indices */
   struct CFLISTITEM *box_lohi;

   /* NetCDF like variable objects */
   struct nccf_var_obj *indices_stt;
   struct nccf_var_obj *weights_stt;
   struct nccf_var_obj *inside_domain_stt;

   /* original grid id */
   int ori_grid_id;

   /* target grid id */
   int tgt_grid_id;
  
   /* interpolation  weights */
   /* Was (double**) but netcdf requires (double*) */
   double *weights;

   /* interpolation indices */
   /* Was (int**) but netcdf requires (int*) */
   int *indices;

   /* number of space dimensions */
   int ndims;

   /* number of target points (product of the target dims) */
   int ntargets;

   /* number of nodes for given cell (should be 2**ndims) */
   int nnodes;

   /* number of valid target values (number of times cell search succeded) */
   int nvalid;

   /* mask array, 1 if inside box, 0 otherwise */
   int *inside_domain;
};


#ifdef __cplusplus
extern "C" {
#endif

  int nccf_def_regrid(int ori_grid_id, int tgt_grid_id, int *regrid_id);

  int nccf_free_regrid(int regrid_id);				  

  int nccf_compute_regrid_weights(int regrid_id, int nitermax, double tolpos,
                                  const int is_periodic[]);

  int nccf_apply_regrid(int regrid_id, int ori_data_id, int tgt_data_id);

  int nccf_inq_regrid_ntargets(int regrid_id, int *ntargets);

  int nccf_inq_regrid_nvalid(int regrid_id, int *nvalid);

  int nccf_inq_regrid_nnodes(int regrid_id, int *nnodes);

  int nccf_add_regrid_forbidden(int regrid_id, const int lo[], const int hi[]);

  int nccf_put_regrid(int ncid, int regrid_id);

  int nccf_def_regrid_from_file(const char *filename,
                                int *regrid_id);
  
#ifdef __cplusplus
}
#endif /* __cplusplus */



#endif /* _NCCF_REGRID_H */
