/**
 * Perform line search in index space
 * $Id: nccf_linesearch_indices.h 632 2011-03-28 18:06:41Z pletzer $
 *
 * \author Alexander Pletzer
 */

int 
nccf_linesearch_indices_TYPE(int ndims, const int dims[], 
			const _TYPE **coordData, 
			const _TYPE targetPos[],
			_TYPE indexTol, 
			const _TYPE dindices[], 
			_TYPE dindices_new[]) {

  int status, i;
  int totErr = 0;

  _TYPE leftIndices[ndims];
  _TYPE righIndices[ndims];
  _TYPE middIndices[ndims];
  _TYPE leftPos[ndims];
  _TYPE middPos[ndims];
  _TYPE righPos[ndims];
  _TYPE leftErr, middErr, righErr, indexErr;

  for (i = 0; i < ndims; ++i) {
    leftIndices[i] = dindices[i];
    righIndices[i] = dindices_new[i];
    middIndices[i] = 0.5 * (leftIndices[i] + righIndices[i]);
  }

  status = nccf_get_position_TYPE(ndims, dims, coordData,
				  leftIndices, leftPos);
  totErr += abs(status);
  status = nccf_get_position_TYPE(ndims, dims, coordData,
				  righIndices, righPos);
  totErr += abs(status);
  status = nccf_get_position_TYPE(ndims, dims, coordData,
				  middIndices, middPos);
  totErr += abs(status);
  
  leftErr = 0.0;
  middErr = 0.0;
  righErr = 0.0;
  indexErr = 0.0;
  for (i = 0; i < ndims; ++i) {
    leftErr += (leftPos[i] - targetPos[i]) * (leftPos[i] - targetPos[i]);
    middErr += (middPos[i] - targetPos[i]) * (middPos[i] - targetPos[i]);
    righErr += (righPos[i] - targetPos[i]) * (righPos[i] - targetPos[i]);
    indexErr += (righIndices[i] - leftIndices[i]) * (righIndices[i] - leftIndices[i]);
    
  }
  leftErr = sqrt(leftErr);
  middErr = sqrt(middErr);
  righErr = sqrt(righErr);
  indexErr = sqrt(indexErr);

  while (indexErr > indexTol) {

    if (leftErr - middErr < righErr - middErr) {
      for (i = 0; i < ndims; ++i) {
	righIndices[i] = middIndices[i];
      }
    }
    else {
      for (i = 0; i < ndims; ++i) {
	leftIndices[i] = middIndices[i];
      }
    }

    for (i = 0; i < ndims; ++i) {
      middIndices[i] = 0.5 * (leftIndices[i] + righIndices[i]);
    }
    
    status = nccf_get_position_TYPE(ndims, dims, coordData,
				    leftIndices, leftPos);
    totErr += abs(status);
    status = nccf_get_position_TYPE(ndims, dims, coordData,
				    righIndices, righPos);
    totErr += abs(status);
    status = nccf_get_position_TYPE(ndims, dims, coordData,
				    middIndices, middPos);
    totErr += abs(status);
  
    leftErr = 0.0;
    middErr = 0.0;
    righErr = 0.0;
    indexErr = 0.0;
    for (i = 0; i < ndims; ++i) {
      leftErr += (leftPos[i] - targetPos[i]) * (leftPos[i] - targetPos[i]);
      middErr += (middPos[i] - targetPos[i]) * (middPos[i] - targetPos[i]);
      righErr += (righPos[i] - targetPos[i]) * (righPos[i] - targetPos[i]);
      indexErr += (righIndices[i] - leftIndices[i]) * (righIndices[i] - leftIndices[i]);
    
    }
    leftErr = sqrt(leftErr);
    middErr = sqrt(middErr);
    righErr = sqrt(righErr);
    indexErr = sqrt(indexErr);
  }

  for (i = 0; i < ndims; ++i) {
    dindices_new[i] = middIndices[i];
  }

  return totErr;
}
